'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var react = require('react');
var reactQuery = require('react-query');
var firestore = require('firebase/firestore');

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
}

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function __generator(thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
}

function usePrevious(value) {
    var ref = react.useRef();
    react.useEffect(function () {
        ref.current = value;
    });
    return ref.current;
}

function useFirestoreDocument(key, ref, options, useQueryOptions) {
    var _this = this;
    var _a, _b;
    var client = reactQuery.useQueryClient();
    var subscribe = (_a = options === null || options === void 0 ? void 0 : options.subscribe) !== null && _a !== void 0 ? _a : false;
    var enabled = (_b = useQueryOptions === null || useQueryOptions === void 0 ? void 0 : useQueryOptions.enabled) !== null && _b !== void 0 ? _b : true;
    var previousRef = usePrevious(ref);
    var isEqual = !!previousRef && ref.id === previousRef.id;
    var unsubscribe = react.useRef();
    // Subscribes to the ref (if enabled) and and if the ref has changed.
    react.useEffect(function () {
        var _a;
        if (enabled && subscribe && !isEqual) {
            unsubscribe.current = firestore.onSnapshot(ref, {
                includeMetadataChanges: (options === null || options === void 0 ? void 0 : options.subscribe)
                    ? (_a = options === null || options === void 0 ? void 0 : options.includeMetadataChanges) !== null && _a !== void 0 ? _a : undefined
                    : undefined,
            }, function (snapshot) {
                client.setQueryData(key, snapshot);
            });
        }
    }, [enabled, subscribe, isEqual, ref]);
    // Unsubscribes the ref subscription when the ref changes.
    react.useEffect(function () {
        if (!isEqual && !!previousRef) {
            return function () {
                var _a;
                (_a = unsubscribe.current) === null || _a === void 0 ? void 0 : _a.call(unsubscribe);
            };
        }
    }, [unsubscribe, isEqual, previousRef]);
    // Unsubscribe when the hook is no longer in use.
    react.useEffect(function () {
        return function () {
            var _a;
            (_a = unsubscribe.current) === null || _a === void 0 ? void 0 : _a.call(unsubscribe);
        };
    }, []);
    return reactQuery.useQuery(key, function () { return __awaiter(_this, void 0, void 0, function () {
        var snapshot;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!((options === null || options === void 0 ? void 0 : options.source) === "cache")) return [3 /*break*/, 2];
                    return [4 /*yield*/, firestore.getDocFromCache(ref)];
                case 1:
                    snapshot = _a.sent();
                    return [3 /*break*/, 6];
                case 2:
                    if (!((options === null || options === void 0 ? void 0 : options.source) === "server")) return [3 /*break*/, 4];
                    return [4 /*yield*/, firestore.getDocFromServer(ref)];
                case 3:
                    snapshot = _a.sent();
                    return [3 /*break*/, 6];
                case 4: return [4 /*yield*/, firestore.getDoc(ref)];
                case 5:
                    snapshot = _a.sent();
                    _a.label = 6;
                case 6: return [2 /*return*/, snapshot];
            }
        });
    }); }, useQueryOptions);
}
function useFirestoreDocumentData(key, ref, options, useQueryOptions) {
    var _a = useQueryOptions || {}, select = _a.select, queryOptions = __rest(_a, ["select"]);
    return useFirestoreDocument(key, ref, options, __assign(__assign({}, queryOptions), { select: function (snapshot) {
            var _a;
            return ((_a = select === null || select === void 0 ? void 0 : select(snapshot)) !== null && _a !== void 0 ? _a : snapshot.data({ serverTimestamps: options === null || options === void 0 ? void 0 : options.serverTimestamps }));
        } }));
}

var namedQueryCache = {};
function isNamedQuery(query) {
    return typeof query === "function";
}
function namedQuery(firestore$1, name) {
    var key = firestore$1.app.name + ":" + name;
    if (namedQueryCache[key]) {
        return namedQueryCache[key];
    }
    return function () {
        return firestore.namedQuery(firestore$1, name).then(function (query) {
            if (query) {
                namedQueryCache[key] = query;
                return query;
            }
            return null;
        });
    };
}
function useFirestoreQuery(key, query, options, useQueryOptions) {
    var _this = this;
    var _a, _b, _c;
    var client = reactQuery.useQueryClient();
    var subscribe = (_a = options === null || options === void 0 ? void 0 : options.subscribe) !== null && _a !== void 0 ? _a : false;
    // Separate the query and named query types.
    var queryFn;
    var namedQueryFn;
    // If the user passed a named query function.
    if (isNamedQuery(query)) {
        // Check whether result is a promise or query.
        if (typeof query === "function") {
            namedQueryFn = query;
        }
        else {
            queryFn = query;
        }
    }
    else {
        queryFn = query;
    }
    var enabled = (_b = useQueryOptions === null || useQueryOptions === void 0 ? void 0 : useQueryOptions.enabled) !== null && _b !== void 0 ? _b : true;
    var previousQuery = usePrevious(queryFn);
    var isEqual = !!previousQuery && firestore.queryEqual(previousQuery, queryFn);
    var _d = react.useState(null), resolvedQuery = _d[0], setResolvedQuery = _d[1];
    var unsubscribe = react.useRef();
    // Anytime the query changes, update the resolved query.
    react.useEffect(function () {
        if (!isEqual && !!queryFn) {
            setResolvedQuery(queryFn);
        }
    }, [isEqual, queryFn]);
    // If a named query is provided, resolve it.
    react.useEffect(function () {
        if (!resolvedQuery && !!namedQueryFn) {
            namedQueryFn().then(setResolvedQuery);
        }
    }, [resolvedQuery, namedQueryFn]);
    // Subscribes to the resolved query.
    react.useEffect(function () {
        var _a;
        if (enabled && resolvedQuery && subscribe) {
            unsubscribe.current = firestore.onSnapshot(resolvedQuery, {
                includeMetadataChanges: (options === null || options === void 0 ? void 0 : options.subscribe)
                    ? (_a = options === null || options === void 0 ? void 0 : options.includeMetadataChanges) !== null && _a !== void 0 ? _a : undefined
                    : undefined,
            }, function (snapshot) {
                client.setQueryData(key, snapshot);
            });
        }
    }, [enabled, resolvedQuery, subscribe]);
    // Unsubscribes the query subscription when the query changes.
    react.useEffect(function () {
        if (!isEqual && !!previousQuery) {
            return function () {
                var _a;
                (_a = unsubscribe.current) === null || _a === void 0 ? void 0 : _a.call(unsubscribe);
            };
        }
    }, [unsubscribe, isEqual, previousQuery]);
    react.useEffect(function () {
        return function () {
            var _a;
            (_a = unsubscribe.current) === null || _a === void 0 ? void 0 : _a.call(unsubscribe);
        };
    }, []);
    return reactQuery.useQuery(key, function () { return __awaiter(_this, void 0, void 0, function () {
        var snapshot;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!((options === null || options === void 0 ? void 0 : options.source) === "cache")) return [3 /*break*/, 2];
                    return [4 /*yield*/, firestore.getDocsFromCache(resolvedQuery)];
                case 1:
                    snapshot = _a.sent();
                    return [3 /*break*/, 6];
                case 2:
                    if (!((options === null || options === void 0 ? void 0 : options.source) === "server")) return [3 /*break*/, 4];
                    return [4 /*yield*/, firestore.getDocsFromServer(resolvedQuery)];
                case 3:
                    snapshot = _a.sent();
                    return [3 /*break*/, 6];
                case 4: return [4 /*yield*/, firestore.getDocs(resolvedQuery)];
                case 5:
                    snapshot = _a.sent();
                    _a.label = 6;
                case 6: return [2 /*return*/, snapshot];
            }
        });
    }); }, __assign(__assign({}, (useQueryOptions || {})), { 
        // If there is a resolved query, use the users option (or default), otherwise it is false.
        enabled: !!resolvedQuery ? (_c = useQueryOptions === null || useQueryOptions === void 0 ? void 0 : useQueryOptions.enabled) !== null && _c !== void 0 ? _c : undefined : false }));
}
function useFirestoreQueryData(key, query, options, useQueryOptions) {
    var _a = useQueryOptions || {}, select = _a.select, queryOptions = __rest(_a, ["select"]);
    return useFirestoreQuery(key, query, options, __assign(__assign({}, queryOptions), { select: function (snapshot) {
            var _a;
            return ((_a = select === null || select === void 0 ? void 0 : select(snapshot)) !== null && _a !== void 0 ? _a : snapshot.docs.map(function (doc) {
                return doc.data({ serverTimestamps: options === null || options === void 0 ? void 0 : options.serverTimestamps });
            }));
        } }));
}

function useFirestoreCollectionMutation(ref, useMutationOptions) {
    return reactQuery.useMutation(function (data) {
        return firestore.addDoc(ref, data);
    }, useMutationOptions);
}
function useFirestoreDocumentMutation(ref, options, useMutationOptions) {
    return reactQuery.useMutation(function (data) {
        if (!!options) {
            return firestore.setDoc(ref, data, options);
        }
        return firestore.setDoc(ref, data);
    }, useMutationOptions);
}

exports.namedQuery = namedQuery;
exports.useFirestoreCollectionMutation = useFirestoreCollectionMutation;
exports.useFirestoreDocument = useFirestoreDocument;
exports.useFirestoreDocumentData = useFirestoreDocumentData;
exports.useFirestoreDocumentMutation = useFirestoreDocumentMutation;
exports.useFirestoreQuery = useFirestoreQuery;
exports.useFirestoreQueryData = useFirestoreQueryData;
